searchData={"items":[{"type":"behaviour","title":"Dumper.Config","doc":"Provides sensible defaults for how data should be rendered.\n\nIf you'd like your own customizations, define a module that implements\nthe `m:Dumper.Config` behaviour:\n\n    # An example dumper config module\n    defmodule MyApp.DumperConfig do\n      use Dumper.Config\n\n      def ids_to_schema() do\n        %{\n          book_id: Library.Book,\n          author_id: Library.Author\n        }\n      end\n\n      def display(%{field: :last_name} = assigns) do\n        ~H| <%= @value %> |\n      end\n\n      def custom_record_links(%Library.Book{} = book) do\n        [\n          {~p\"/logging/#{book.id}\", \"Logs\"},\n          {\"https://goodreads.com/search?q=#{book.title}\", \"Goodreads\"},\n        ]\n      end\n    end\n\nThen update the `live_dashboard` entry in the `router.ex` file to add the config module:\n\n    live_dashboard \"/dashboard\", additional_pages:\n      [dumper: {Dumper.LiveDashboardPage, repo: MyApp.Repo, config_module: MyApp.DumperConfig}]\n\nImplementing each callback provides a different way to control how the data is rendered:\n- `c:ids_to_schema/0`: turn id values into clickable links\n- `c:display/1`: define a functional component for complete control over how fields are rendered\n- `c:custom_record_links/1`: display custom links when viewing an indivisual record\n- `c:Dumper.Config.additional_associations/1`: display custom associations not defined in the Ecto schema\n- `c:Dumper.Config.allowed_fields/0`: any fields not included will be ignored\n- `c:Dumper.Config.excluded_fields/0`: any fields included will be ignored\n- `c:Dumper.Config.large_tables/0`: tables included will not render the total record count and won't sort by inserted_at\n\nThe `use Dumper.Config` brings in the default definitions of behaviour, so you can\nchoose to define one, all, or none of them.  As such, even this is a valid implementation\n(although it would be functionally the same as not defining a config module at all):\n\n    defmodule MyApp.DumperConfig do\n      use Dumper.Config\n    end","ref":"Dumper.Config.html"},{"type":"callback","title":"Dumper.Config.additional_associations/1","doc":"Additional records to be rendered on the given record's page.\n\n    @impl Dumper.Config\n    def additional_associations(%Book{id: book_id}) do\n      # look up reviews on goodreads\n      [goodreads_reviews: [top_review, lowest_review]]\n    end\n\nFor a given record, return a keyword list of association name to list of records.  Overriding\nthis callback allows you to render more data at the bottom of the page.  This is useful\nwhen the given record doesn't explicitly define an association, or the data you want to\ndisplay doesn't live in the database.\n\nNote that while regular associations are paginated, since these are custom we can't\nautomatically paginate them.  It's recommended to cap the number of records returned.","ref":"Dumper.Config.html#c:additional_associations/1"},{"type":"callback","title":"Dumper.Config.allowed_fields/0","doc":"A mapping from schema module => list of its fields that will be rendered.\n\nCan return\n- `nil`: all fields in all tables are shown\n- a map: a field is only displayed if the exact Schema + field pairing exists in the map\n- `{map, :strict}`: a field is only displayed if the exact Schema + field pairing exists in the map\n- `{map, :lenient}`: a field is displayed if the exact Schema + field pairing exists in the map **or** the schema module is not present in the map\n\nHere's an example where we return a mapping of schema modules to the fields we want to allow displayed:\n\n    @impl Dumper.Config\n    def allowed_fields() do\n      %{\n        Patron => [:id, :last_name],\n        Book => [:title]\n      }\n    end\n\nIn the above example, the returned map defaults to strict mode.  Rendered fields would\ninclude `patron.last_name` and `book.title`.  Hidden fields would include fields like\n`patron.first_name` and `author.last_name`.\n\n    @impl Dumper.Config\n    def allowed_fields() do\n      map = %{\n        Patron => [:id, :last_name],\n        Book => [:title]\n      }\n      {map, :lenient}\n    end\n\n\nIn the above example, `lenient` strictness means that `author.last_name` would now be rendered even though the `Author` key not explicitly defined in the returned mapping.\n\nIt's recommended to at least include the primary key field in the list so that there is at least\none field to display.\n\n`c:excluded_fields/0` is ignored if:\n- this callback is implemented and returns strict mode (or returns only a map)\n- this callback is implemented and returns lenient mode, but the schema key is present in the map","ref":"Dumper.Config.html#c:allowed_fields/0"},{"type":"callback","title":"Dumper.Config.custom_record_links/1","doc":"Custom links rendered when viewing a specific record.\n\nThis function takes a record you can pattern match on, and must return a list of `{route, text}`\ntuples.\n\n    @impl Dumper.Config\n    def custom_record_links(%Book{} = book) do\n      [\n        {~p\"/logging/#{book.id}\", \"Logs\"},\n        {\"https://goodreads.com/search?q=#{book.title}\", \"Goodreads\"},\n      ]\n    end\n\n    def custom_record_links(%Ticket{} = ticket),\n      do: [{\"https://jira.com/#{ticket.project}/#{ticket.id}\", \"Jira\"}]\n\nIn the above example, any `Book` record you visit in the Dumper will display two links labelled\n\"Logs\" and \"Goodreads\" at the top of the page.  Any `Ticket` record will likewise display one\nlink, \"Jira\", in that spot.  Routes can be internal or external, verified routes, or plain strings.\n\nLogs, dashboards, traces, support portals, etc are all common use cases, but any `{route, text}`\npair is possible.","ref":"Dumper.Config.html#c:custom_record_links/1"},{"type":"callback","title":"Dumper.Config.display/1","doc":"Fine-grained control over how any field is rendered.  This is a functional component that takes\nin an `assigns` map and returns a valid `heex` expression.\n\nBy default, Dumper has some sensible defaults for how redacted fields, values like `true`, `false`, `nil`, and data types like dates and datetimes are rendered.\n\nIt is useful to define as many `c:display/1` function heads as you want, pattern matching on specific values to pick out the specific ones you'd like to customize.\n\nThe `assigns` that is passed in is a map of the following form:\n\n    # assigns\n    %{\n      module: Library.Author,\n      field: :last_name,\n      resource: %Library.Author{ ... }, # the entire ecto struct\n      value: \"Smith\",\n      type: :binary, # the Ecto data type\n      redacted: false\n    %}\n\nSo for example, if you wanted every last name to be red except for the Author table, which should have blue last names, you could do the following:\n\n    @impl Dumper.Config\n    def display(%{field: :last_name, module: Library.Author} = assigns) do\n      ~H| <%= @value %> |\n    end\n\n    def display(%{field: :last_name} = assigns) do\n      ~H| <%= @value %> |\n    end\n\nIn this way, you can have near complete control over how a particular field, data type, or entire module is displayed.\n\nNote that LiveDashboard ships with [Bootstrap 4.6](https://getbootstrap.com/docs/4.6), so you are free to use Bootstrap classes in your styling to help achieve a consistent look and feel.","ref":"Dumper.Config.html#c:display/1"},{"type":"callback","title":"Dumper.Config.excluded_fields/0","doc":"A mapping from schema module => list of its fields that will be excluded from being rendered.\n\n    @impl Dumper.Config\n    def excluded_fields() do\n      %{\n        Library.Patron => [:email_address, :date_of_birth],\n        Library.Book => [:purchase_price]\n      }\n    end\n\nIn the above example, when displaying any patron record, the `email_address` and `date_of_birth`\nfields will not be rendered.  All others fields will be displayed.  The `email_address` and\n`date_of_birth` fields will not even be sent down through the `c:display/1` callback.  The\nsame for any book record; the `purchase_price` field will never be rendered.  All other\nschemas are unaffected - for example a `Author` schema would display all of its fields, even\nthough it is not included in the returned map.\n\nThe excluded fields will only hide fields for a module if the module exists as a key in the\nreturned map.\n\nIt's recommended to at least include the primary key field in the list so that there is at least\none field to display.\n\nSee `c:allowed_fields/0` for cases where `c:excluded_fields/0` is ignored.","ref":"Dumper.Config.html#c:excluded_fields/0"},{"type":"callback","title":"Dumper.Config.ids_to_schema/0","doc":"A map of ids (as atoms) to the schema module they should link to.\n\nEach key/value pair in the map will automatically render as a clickable link that\nnavigates the user to the dumper page for that specific record.\n\nBy default this map is empty.\n\nExample:\n\n    def ids_to_schema() do\n      %{\n        book_id: Library.Book,\n        author_id: Library.Author\n      }\n    end\n\nHere, any field in any schema named `book_id` would render as a link to the `Book` record,\nvia a `Repo.get!(id)` call under the hood, instead of just printing the value.  This allows\nyou to easily navigate through your data by clicking connected links.","ref":"Dumper.Config.html#c:ids_to_schema/0"},{"type":"callback","title":"Dumper.Config.large_tables/0","doc":"Tables so large they timeout when querying the row count.\n\nBy including schema modules in this list, Dumper will omit querying and displaying of the\ntotal number of entries and order by `id` instead of `inserted_at`.  A clue you may need\nto add a table to this list is if the page errors out when rendering the list of records.\n\n    @impl Dumper.Config\n    def large_tables, do: [Book, Patrons]","ref":"Dumper.Config.html#c:large_tables/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.2.7 - Changelog","doc":"","ref":"changelog.html#v0-2-7"},{"type":"extras","title":"Improvements - Changelog","doc":"* Added `large_tables/0` optional config to handle performance issues for tables with large row counts.  [https://github.com/adobe/elixir-dumper/issues/7](Issue #7) showed that large tables would time out when rendering the records page due to the expensive count query.  It was also discovered the order by inserted_at could also contribute to timeouts.","ref":"changelog.html#improvements"},{"type":"extras","title":"Bug Fix - Changelog","doc":"* Addressed [https://github.com/adobe/elixir-dumper/issues/7](Issue #7) by removing total number of entries altogether.","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v0.2.6 - Changelog","doc":"","ref":"changelog.html#v0-2-6"},{"type":"extras","title":"Improvements - Changelog","doc":"* Removed unnecessary `action` url parameter.  The correct page to render can be derived from the presence of the module and id parameters.","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.2.5 - Changelog","doc":"","ref":"changelog.html#v0-2-5"},{"type":"extras","title":"Improvements - Changelog","doc":"* Was previously loading all associations, which caused timeouts for records with large amounts of associated records.\n* Tests are now run on PRs instead of only pushes against main.","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.2.4 - Changelog","doc":"","ref":"changelog.html#v0-2-4"},{"type":"extras","title":"Improvements - Changelog","doc":"* Correct the `Dumper.Config` callback specs.","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.2.3 - Changelog","doc":"","ref":"changelog.html#v0-2-3"},{"type":"extras","title":"Improvements - Changelog","doc":"* Allow override of the auto-discovery of the `otp_app`.","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.2.2 - Changelog","doc":"","ref":"changelog.html#v0-2-2"},{"type":"extras","title":"Improvements - Changelog","doc":"* Updated the package description to reflect that it is now a LiveDashboard page and not a mix generator.","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.2.1 - Changelog","doc":"","ref":"changelog.html#v0-2-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Search for a record by id\n* Use router config instead of app config\n* Update links to use `<.link navigate={}>` instead of firing events","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Changed from mix task to a LiveDashboard plugin","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.1.1 - Changelog","doc":"","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Fix phoenix heex warning where name attribute was set to \"id\".  Renamed to \"search_id\".","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Initial release of Dumper","ref":"changelog.html#improvements"},{"type":"extras","title":"Dumper","doc":"[![Hex.pm](https://img.shields.io/hexpm/v/dumper)](https://hex.pm/packages/dumper)\n[![Hexdocs.pm](https://img.shields.io/badge/docs-hexdocs.pm-purple)](https://hexdocs.pm/dumper)\n[![Github.com](https://github.com/adobe/elixir-dumper/actions/workflows/ci.yml/badge.svg)](https://github.com/adobe/elixir-dumper/actions)\n\n# Dumper\n\n_Takes your data and dumps it to the screen!_\n\nDumper uses reflection to find all your app's ecto schemas and provide routes to browse their data.  It's packaged as a Live Dashboard plugin for easy navigation.\n\n![dumper](assets/dumper.gif)","ref":"readme.html"},{"type":"extras","title":"Requirements - Dumper","doc":"Dumper only works with Phoenix 1.7+ applications that use Ecto.","ref":"readme.html#requirements"},{"type":"extras","title":"About - Dumper","doc":"Dumper aims to make it as easy as possible for everyone on a project to access and understand its data.\n\n- All ids can be linked, so it's an incredibly fast way to explore a data model against real data.\n- Because it's implemented with reflection, it automatically covers every schema module in your project.\n- Styling is kept consistent with the LiveDashboard theme so that data is front and center.\n- Associations are also shown for a given record, all on the same page.\n- Non-intrusive. No changes necessary to existing modules/files.\n- Read-only.  No accidentally deleting or editing data while browsing.\n- Shareable URLs. Having a shareable link to every record in your database is very useful for debugging. It gives everyone including non-technical teammates the ability to get a better understanding of your data model.","ref":"readme.html#about"},{"type":"extras","title":"Installation and Usage - Dumper","doc":"Add `dumper` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:dumper, \"~> 0.2.6\"}\n  ]\nend\n```\n\nInstall and configure [Phoenix Live Dashboard](https://hexdocs.pm/phoenix_live_dashboard) if you haven't already.  Then modify `router.ex` to include the `dumper` as a plugin:\n\n``` elixir\nlive_dashboard \"/dashboard\", additional_pages: [dumper: {Dumper.LiveDashboardPage, repo: MyApp.Repo}]\n```\n\nBy default, Dumper will auto-discover all Ecto schemas associated with the provided repo's `otp_app`.  You can override this behavior by explicitly passing an app as an option with `otp_app: :my_app`.\n\nYou can now run your web app, navigate to dumper tab within the live dashboard, and view all your data.","ref":"readme.html#installation-and-usage"},{"type":"extras","title":"Customization - Dumper","doc":"","ref":"readme.html#customization"},{"type":"extras","title":"Config Module - Dumper","doc":"It is *highly recommended* to customize the `dumper`.  To do so, you can optionally define a module that implements the `Dumper.Config` behavior.  Add it to the `live_dashboard` entry in your `router.ex` file:\n\n``` elixir\nlive_dashboard \"/dashboard\", additional_pages:\n  [dumper: {Dumper.LiveDashboardPage, repo: MyApp.Repo, config_module: MyApp.DumperConfig}]\n```\n\nHere's an example config module:\n\n``` elixir\ndefmodule MyApp.DumperConfig do\n  use Dumper.Config\n\n  @impl Dumper.Config\n  def ids_to_schema() do\n    %{\n      book_id: Library.Book,\n      author_id: Library.Author\n    }\n  end\n\n  def excluded_fields() do\n    %{\n      Library.Employee => [:salary, :email_address],\n      Library.Book => [:price]\n    }\n  end\n\n  def display(%{field: :last_name} = assigns) do\n    ~H| <%= @value %> |\n  end\n\n  def custom_record_links(%Library.Book{} = book) do\n    [\n      {\"https://goodreads.com/search?q=#{book.title}\", \"Goodreads\"},\n      {~p\"/logging/#{book.id}\", \"Logs\"}\n    ]\n  end\nend\n\n```\n\nCommon customizations are using the `c:Dumper.Config.ids_to_schema/0` to turn id values into clickable links and using `c:Dumper.Config.custom_record_links/1` to provide extra links to useful information when viewing a specific record.\n\nTake a look a `c:Dumper.Config.ids_to_schema/0`, `c:Dumper.Config.allowed_fields/0`, `c:Dumper.Config.excluded_fields/0`, `c:Dumper.Config.display/1`, `c:Dumper.Config.additional_associations/1`, and `c:Dumper.Config.custom_record_links/1` for more information on how each optional callback lets you customize how your data is rendered.","ref":"readme.html#config-module"},{"type":"extras","title":"Other notes - Dumper","doc":"","ref":"readme.html#other-notes"},{"type":"extras","title":"Rendering Embeds - Dumper","doc":"The index page and association tables on the show page by default omit columns that are embeds.  This is purely for display purposes, as those values tend to take up a lot of vertical space.  This is currently not configurable, but may be in the future.","ref":"readme.html#rendering-embeds"},{"type":"extras","title":"Redactions - Dumper","doc":"By default, schema fields with `redact: true` are hidden and replaced with the text `redacted`.  If you're running the Dumper in a non-production environment or against dummy data, you may want to disregard the redacted fields.  To do that, you can add a `display/1` function head like the following:\n\n``` elixir\ndef display(%{redacted: true} = assigns), do: ~H|<%= @value %>|\n```\n\nYou can refine that down to a specific schema and/or field as well by pattern matching the assigns.","ref":"readme.html#redactions"},{"type":"extras","title":"Security - Dumper","doc":"As with LiveDashboard more broadly, it is highly recommended that you put the route behind some sort of [admin authentication](https://hexdocs.pm/phoenix_live_dashboard/Phoenix.LiveDashboard.html#module-extra-add-dashboard-access-on-all-environments-including-production) if you want to use `dumper` in production.\n\nThe `c:Dumper.Config.allowed_fields/0` and `c:Dumper.Config.excluded_fields/0`callbacks are another way to be explicit about what data is shown or hidden altogether.\n\nYou could also hide the plugin altogether by modifying the live_dashboard route.  For example, this would require a `:dumper` `enabled: true` config to be set in order to display the `dumper` tab in the live dashboard:\n\n``` elixir\nlive_dashboard \"/dashboard\",\n  additional_pages: [] ++ (if Application.get_env(:dumper, :enabled, false), do: [dumper: {Dumper.LiveDashboardPage, repo: MyApp.Repo}], else: [])\n```\n\nThis would allow you to configure showing or hiding the `dumper` based on environments.","ref":"readme.html#security"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}